extern crate self as i18n;

use std::{collections::HashMap, sync::OnceLock};

pub mod message_ids;
pub mod en_us;
pub mod zh_cn;

static I18N: OnceLock<HashMap<&'static str, HashMap<&'static str, &'static str>>> = OnceLock::new();

fn init_i18n() -> HashMap<&'static str, HashMap<&'static str, &'static str>> {
    let mut i18n_map = HashMap::new();
    i18n_map.insert(en_us::LANGUAGE_ID, en_us::MESSAGE.clone());
    i18n_map.insert(zh_cn::LANGUAGE_ID, zh_cn::MESSAGE.clone());
    i18n_map
}

fn get_i18n() -> &'static HashMap<&'static str, HashMap<&'static str, &'static str>> {
    I18N.get_or_init(init_i18n)
}

pub fn lang(lang_id: &'static str, message_id: &'static str) -> &'static str {
    get_i18n()
        .get(lang_id)
        .and_then(|map| map.get(message_id))
        .unwrap_or_else(|| &"UNKNOWN")
}

pub fn convert_message_with_params(message_template: &'static str, args: Vec<&str>) -> String {
    if message_template.contains("{}") {
        let message_split_array: Vec<&str> = message_template.split("{}").collect();
        // tracing::info!("message_template: {:?}, args: {:?}", message_split_array, args);
        let mut message = String::new();
        for (index, message_ele) in message_split_array.iter().enumerate() {
            message += message_ele;
            if index < args.len() && index != message_split_array.len() - 1 {
                message += args[index];
            }
        }
        message
    } else {
        String::from(message_template)
    }
}

#[macro_export]
macro_rules! message {
    ($lang_id:expr, $message_id:expr) => {
        i18n::lang($lang_id, $message_id)
    };
    ($lang_id:expr, $message_id:expr, $($arg:expr),*) => {
        {
            let message_template: &'static str = i18n::lang($lang_id, $message_id);
            let mut args: Vec<&str> = vec![];
            $(
                args.push($arg);
            )*
            i18n::convert_message_with_params(message_template, args)
        }
    };
}
